/*
 * ===== global constants =================================
 */

// URLs
var ctFrontendRootURL = "http://appuntat";
var ctBackendRootURL = "http://appuntat:8888";

// profile types
var ctProfileTypeSportist = "PROFILE_TYPE_SPORTIST";
var ctProfileTypeLocation = "PROFILE_TYPE_LOCATION";
var ctProfileTypeEvent = "PROFILE_TYPE_EVENT";

// global value keys
var ctKeySessionId = "GVKEY_SESSION_ID";
var ctKeyUserId = "GVKEY_USER_ID";
var ctKeyProfileId = "GVKEY_PROFILE_ID";
var ctKeyProfileType = "GVKEY_PROFILE_TYPE";

/*
 * ===== global functions =================================
 */

// Sets a value globally availabe, assigned to the given key
function gfGetGlobalValue( key ){
    if(typeof(Storage) !== "undefined") {
        return localStorage.getItem(key);
    } else {
        console.error("appuntat.global.js: gfGetGlobalValue: No LocalStorage available!");
    }
}

// Gets the globally available value assigned to the given key
function gfSetGlobalValue( key, value ){
    if(typeof(Storage) !== "undefined") {
        localStorage.setItem(key,value);
    } else {
        console.error("appuntat.global.js: gfGetGlobalValue: No LocalStorage available!");
    }
}

// Clears the global values
function gfClearGlobalValues(){
    localStorage.clear();
}

// Returns a JSON representation of the given query string
function gfQueryStringToJSON(pQueryString){
    var pairs = pQueryString.split('&');
    var result = {};
    pairs.forEach(function(pair){
        pair = pair.split('=');
        result[pair[0]] = decodeURIComponent(pair[1] || '');
    });
    return JSON.parse(JSON.stringify(result));
}

// Returns a JSON object containing the params in the current URL
function gfGetCurrentUrlParams(){
    var paramsString = location.search.slice(1);
    if(paramsString.length < 1) 
        return null;
    else 
        return gfQueryStringToJSON( paramsString );
}

// Show a result message
// Requires: 
// - a modal div called "mdlMessage", with a label "lblMessage" in it
function gfShowMessage( message, isError ){
    if( $('#mdlProgress').length ){
        $('#mdlProgress').foundation('reveal', 'close');
    }
    $('#lblMessage').html( message );    
    if( isError ){
        $('#lblMessage').addClass("error");
    }else{
        $('#lblMessage').removeClass("error");
    }
    $('#mdlMessage').foundation('reveal', 'open');
}

// Parses a given datetime in string format (dd/mm/yyyy hh:mm) into a JSON object
// Returns:
//    - a JSON object containing {date, time, day, month, year, hour, minute}
function gfParseDateTime( strDateTime ){
    var dateTime = null;
    if( strDateTime != null ){
        var dateAndTime = strDateTime.split(" ");
        if(dateAndTime.length == 2){
            var date = dateAndTime[0].split("/");
            var time = dateAndTime[1].split(":");
            if( date.length == 3 && time.length == 2 ){
                dateTime = { "date":dateAndTime[0], "time":dateAndTime[1], "day":date[0], "month":date[1], "year":date[2], "hour":time[0], "minute":time[1] };
            }
        }
    }
    if(dateTime == null) console.error('gfParseDateTime(): Returning NULL dateTime because input string {'+strDateTime+'} is invalid.');
    return dateTime;
}

// Load the list of profiles into the given target table
// Params:
//    - targetTableId : ID of the table where the result will be loaded
//    - listedProfileType: Type of the profile entities to be listed (ctProfileTypeSportist, ...)
//    - listedColumns: array of string with all the columns of the target table
//    - onSelectCallback: function to be invoked when an element of the list is clicked, matching the format
//        callback( elementId, elementName )
//    - relatedProfileType / relatedProfileId: if provided, lists only the profiles related to the given profile (type/id)
// Requires:
//  - the table "targetTableId"
function gfLoadProfileList( targetTableId, listedProfileType, listedColumns, onSelectCallback, relatedProfileType, relatedProfileId ){
    var profileToList = "Undefined";
    if(listedProfileType == ctProfileTypeEvent){
        profileToList = "Event";
    }if(listedProfileType == ctProfileTypeSportist){
        profileToList = "Sportist";
    }if(listedProfileType == ctProfileTypeLocation){
        profileToList = "Location";
    }
    var ajaxRequest = ctBackendRootURL + "/list" + profileToList;
    if( relatedProfileType !== undefined && relatedProfileId !== undefined ){
        ajaxRequest += "?profileType=".concat(relatedProfileType);
        ajaxRequest += "&profileId=".concat(relatedProfileId);
    }
    gfLoadTableData( targetTableId, listedColumns, ajaxRequest, onSelectCallback );
}

// Performs an AJAX request and loads the result in the given table
// Params:
//    - targetTableId : ID of the table where the result will be loaded
//    - listedColumns: array of string with the column names in the target table
//    - onSelectCallback: function to be invoked when an element of the list is clicked, matching the format
//        callback( elementId, elementName )
// Requires:
//  - the table "targetTableId"
function gfLoadTableData( targetTableId, columnNames, ajaxRequest, onSelectCallback ){
    
    var tableId = '#' + targetTableId ;
    
    var tableColumns = new Array();
    for( columnIdx in columnNames ){
        tableColumns[ tableColumns.length ] = { "data": columnNames[ columnIdx ] };
    }
    
    $(tableId).dataTable({
        "destroy": true,
        "processing": true,
        "serverSide": true,
        "ajax": ajaxRequest,
        "columns": tableColumns,
        "columnDefs": [
            {
                "targets": [0],
                "visible": false
            }
        ]            
    });
    
    $(tableId + ' tbody').on('click', 'tr', function () {
        var selectedId = $(tableId).DataTable().cell( this, 0 ).data();
        var selectedName = $(tableId).DataTable().cell( this, 1 ).data();
        onSelectCallback( selectedId, selectedName );
    });    
}

// Parses a given datetime string "dd/mm/yy hh:mm" into a JSON {day,month,year,hour,minute}
function gfParseDateTime( dateTime ){
    if( dateTime != undefined && dateTime.length>0 ){
        var dateAndTime = dateTime.split(" ");
        if( dateAndTime.length == 2){            
            var date = dateAndTime[0].split("/");
            var time = dateAndTime[1].split(":");
            if( date.length==3 && time.length==2 ){
                return { "day":date[0],
                         "month": date[1],
                         "year": date[2],
                         "hour": time[0],
                         "minute": time[1]
                };
            }
        }
    }
    return undefined;
}